/*
Copyright 2017-2018 Aron Barath

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

3. Neither the name of the copyright holder nor the names of its contributors
may be used to endorse or promote products derived from this software without
specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/

#include "ncbricks.h"
#include <stdlib.h>
#include <string.h>
#include <assert.h>

void missile_show_all()
{
	size_t i;

	for(i=0;i<g_missiles.size;++i)
	{
		missile_show((struct missile*)g_missiles.pointers[i]);
	}
}

void missile_hide_all()
{
	size_t i;

	for(i=0;i<g_missiles.size;++i)
	{
		missile_hide((struct missile*)g_missiles.pointers[i]);
	}
}

void missile_show(const struct missile* const missile)
{
	if(g_use_colors)
	{
		wattron(g_game_win, COLOR_PAIR(CPAIR_MISSILE));
	}

	mvwaddch(g_game_win, g_board_top+missile->pos.y, g_board_left+missile->pos.x, '!'|A_BOLD);

	if(g_use_colors)
	{
		wattroff(g_game_win, COLOR_PAIR(CPAIR_MISSILE));
	}
}

void missile_hide(const struct missile* const missile)
{
	mvwaddch(g_game_win, g_board_top+missile->pos.y, g_board_left+missile->pos.x, ' ');
}

void missile_delete_all()
{
	while(0<g_missiles.size)
	{
		struct missile* const missile = (struct missile*)g_missiles.pointers[--g_missiles.size];
		missile_hide(missile);
		free(missile);
	}
}

static void spawn(const int x, const int y)
{
	struct missile* const missile = (struct missile*)malloc(sizeof(struct missile));
	assert(NULL!=missile);
	memset(missile, 0, sizeof(struct missile));

	missile->pos.x = x;
	missile->pos.y = y;
	ptrarr_add(&g_missiles, missile);
	missile_show(missile);
}

void missile_spawn(const struct paddle* const paddle)
{
	spawn(paddle->x + (paddle->width / 2), g_paddle_y - 1);
}

void missile_spawn_apocalypse()
{
	size_t i;

	for(i=0;i<BRICKS_PER_ROW;++i)
	{
		spawn((i*BRICK_WIDTH)+(BRICK_WIDTH/2), g_paddle_y - 1);
	}
}
